@@ -33,19 +33,14 @@ from pre_workbench.guihelper import navigateBrowser
 from pre_workbench.app import NavigateCommands, GlobalEvents
 from pre_workbench import configs
 # noinspection PyUnresolvedReferences
-from pre_workbench.windows.content import textfile
-# noinspection PyUnresolvedReferences
-from pre_workbench import typeeditor
-# noinspection PyUnresolvedReferences
 from pre_workbench import windows
 from pre_workbench.configs import getIcon
 from pre_workbench.datawidgets import DynamicDataWidget
+from pre_workbench.util import get_app_version
 from pre_workbench.windows.dockwindows import FileBrowserWidget, MdiWindowListWidget, StructInfoTreeWidget, \
-	StructInfoCodeWidget, DataInspectorWidget
+	StructInfoCodeWidget, DataInspectorWidget, BinwalkDockWidget, ExtToolDockWidget, SearchDockWidget
 from pre_workbench.windows.dockwindows import RangeTreeDockWidget, RangeListWidget, SelectionHeuristicsConfigWidget, LogWidget
 from pre_workbench.controls.genericwidgets import MemoryUsageWidget, showPreferencesDlg
-# noinspection PyUnresolvedReferences
-from pre_workbench.windows.content.objectwindow import ObjectWindow
 from pre_workbench.typeeditor import JsonView
 from pre_workbench.typeregistry import WindowTypes
 from pre_workbench.windows.content.hexfile import HexFileWindow
@@ -63,7 +58,7 @@ class WorkbenchMain(QMainWindow):
 		self.project = project
 		self.mappedChildActions = list()
 		self.curChildMeta = dict()
-		self.initUI()
+		self._initUI()
 		self.restoreChildren()
 		self.mdiArea.restoreState(self.project.getValue("DockState", b""))
 
@@ -300,27 +295,47 @@ class WorkbenchMain(QMainWindow):
 
 	def _initDockWindows(self):
 		self.dockWidgets = {}
+
 		self.createDockWnd("Project Files", "folder-tree.png", FileBrowserWidget(self.project.projectFolder), ads.LeftDockWidgetArea, showFirstRun=True)
 		self.dockWidgets["Project Files"].on_open.connect(self.openFile)
+
 		self.createDockWnd("Window List", "applications-stack.png", MdiWindowListWidget(), ads.LeftDockWidgetArea)
+
 		self.zoomWindow = DynamicDataWidget()
 		self.zoomWindow.meta_updated.connect(self.onMetaUpdateRaw)
-		self.createDockWnd("Zoom", "document-search-result.png", self.zoomWindow, ads.BottomDockWidgetArea)
-		self.zoom_updated.connect(lambda content: self.zoomWindow.setContents(content))
+		self.createDockWnd("Zoom", "document-search-result.png", self.zoomWindow, ads.BottomDockWidgetArea, showFirstRun=True)
+		self.zoom_updated.connect(lambda content: self.zoomWindow.setContents(content) if content is not None else None)
+
 		self.createDockWnd("Data Inspector", "user-detective-gray.png", DataInspectorWidget(), ads.BottomDockWidgetArea, showFirstRun=True)
 		self.selected_bytes_updated.connect(self.dockWidgets["Data Inspector"].on_select_bytes)
+
 		self.createDockWnd("Data Source Log", "terminal--exclamation.png", LogWidget("DataSource"), ads.TopDockWidgetArea)
 		self.createDockWnd("Application Log", "terminal--exclamation.png", LogWidget(""), ads.TopDockWidgetArea)
 
 		self.createDockWnd("Grammar Definition Tree", "tree.png", StructInfoTreeWidget())
 		self.createDockWnd("Grammar Definition Code", "tree--pencil.png", StructInfoCodeWidget(), showFirstRun=True)
+
 		self.createDockWnd("Grammar Parse Result", "tree--arrow.png", RangeTreeDockWidget(), showFirstRun=True)
 		self.meta_updated.connect(self.dockWidgets["Grammar Parse Result"].on_meta_update)
-		self.createDockWnd("Selected Ranges", "bookmarks.png", RangeListWidget(), showFirstRun=True)
+
+		self.createDockWnd("Selected Ranges", "bookmarks.png", RangeListWidget())
 		self.meta_updated.connect(self.dockWidgets["Selected Ranges"].on_meta_update)
+
 		self.createDockWnd("Selection Heuristics", "clipboard-task.png", SelectionHeuristicsConfigWidget())
 
-	def initUI(self):
+		self.createDockWnd("Binwalk", "user-detective-gray.png", BinwalkDockWidget(), ads.BottomDockWidgetArea, showFirstRun=False)
+		self.selected_bytes_updated.connect(self.dockWidgets["Binwalk"].on_select_bytes)
+
+		self.createDockWnd("External Tools", "user-detective-gray.png", ExtToolDockWidget(), ads.BottomDockWidgetArea, showFirstRun=False)
+		self.selected_bytes_updated.connect(self.dockWidgets["External Tools"].on_select_bytes)
+
+		self.createDockWnd("Search", "user-detective-gray.png", SearchDockWidget(), ads.BottomDockWidgetArea, showFirstRun=False)
+		self.meta_updated.connect(self.dockWidgets["Search"].on_meta_updated)
+
+
+
+
+	def _initUI(self):
 		ads.CDockManager.setConfigFlag(ads.CDockManager.FocusHighlighting, True)
 		ads.CDockManager.setConfigFlag(ads.CDockManager.MiddleMouseButtonClosesTab, True)
 		ads.CDockManager.setConfigFlag(ads.CDockManager.AllTabsHaveCloseButton, True)
@@ -361,7 +376,8 @@ class WorkbenchMain(QMainWindow):
 		self.show()
 
 	def showAboutBox(self):
-		QMessageBox.about(self, "PRE Workbench", "Protocol Reverse Engineering Workbench\n\n"
+		QMessageBox.about(self, "PRE Workbench", "Protocol Reverse Engineering Workbench\n"
+												 "Version "+get_app_version()+"\n\n"
 												 "Copyright (c) 2022 Mira Weller\n"
 												 "Licensed under the GNU General Public License Version 3\n\n"
 												 "https://github.com/luelista/pre_workbench")
